@@ -25,6 +25,7 @@ USE_COLOR = False
 def main():
     global MOVED_BACK, USE_COLOR
     options = parse_args()
+
     if options.upgrade:
         return autoupgrade()
     if options.pretty:
@@ -37,7 +38,7 @@ def main():
     elif options.grep:
         print grep(add_os_thread_info(options.pid, get_stack_trace(options)), options.grep)
     elif options.sample:
-        print sample(options.pid, 4, 10, int(float(options.sample) / float(10)))
+        print sample(options.pid, 4, 10, int(float(options.sample) / float(10)), options.jstack)
     else:
         sys.argv = [sys.argv[0], '--help']
         print "No action specified"
@@ -46,7 +47,7 @@ def main():
 
 def get_stack_trace(options):
     if options.pid:
-        return get_stack_trace_from_pid(options.pid)
+        return get_stack_trace_from_pid(options.pid, options.jstack)
     elif options.input:
         return get_stack_trace_from_file(options.input)
     else:
@@ -58,21 +59,35 @@ def get_stack_trace(options):
 def get_stack_trace_from_file(filename):
     if filename.strip() == '-':
         return sys.stdin.read()
+    elif filename.startswith('http:'):
+        return urllib.urlopen(filename).read()
     else:
         with open(filename) as f:
             return f.read()
 
 
-def get_stack_trace_from_pid(pid):
-    with tempfile.NamedTemporaryFile() as stackfile:
-        try:
-            MOVED_BACK = False
-            move_stdout(pid, stackfile.name)
-            os.kill(pid, 3)
-            return read_stack_trace(stackfile)
-        finally:
-            if not MOVED_BACK:
-                move_stdout(pid, edge=END)
+def get_stack_trace_from_pid(pid, use_jstack):
+    if use_jstack:
+        return Popen('{jstack} {pid}'.format(jstack=find_jstack(), pid=pid), stdout=PIPE, stdin=DEVNULL, shell=True).communicate()[0]
+    else:
+        with tempfile.NamedTemporaryFile() as stackfile:
+            try:
+                MOVED_BACK = False
+                move_stdout(pid, stackfile.name)
+                os.kill(pid, 3)
+                return read_stack_trace(stackfile)
+            finally:
+                if not MOVED_BACK:
+                    move_stdout(pid, edge=END)
+
+
+def find_jstack():
+    default = Popen("which jstack", stdout=PIPE, stdin=DEVNULL, shell=True).communicate()[0]
+    if default.strip():
+        return default.strip()
+    if os.environ.get('JAVA_HOME'):
+        return os.path.join(os.environ['JAVA_HOME'], 'bin', 'jstack')
+    raise RuntimeError("Could not find jstack - do you have it installed in $JAVA_HOME?")
 
 
 def read_stack_trace(stackfile):
@@ -188,7 +203,7 @@ def grep(stacktrace, text):
     return '\n\n'.join(colorize_thread(thread) for thread in threads)
 
 
-def sample(pid, nlines, samples, wait_time):
+def sample(pid, nlines, samples, wait_time, use_jstack):
     sys.stdout.write("Sampling.")
     sys.stdout.flush()
     thread_runnable_counts = {}
@@ -196,7 +211,7 @@ def sample(pid, nlines, samples, wait_time):
     for _ in range(samples):
         sys.stdout.write(".")
         sys.stdout.flush()
-        threads = split_threads(add_os_thread_info(pid, get_stack_trace_from_pid(pid)))
+        threads = split_threads(add_os_thread_info(pid, get_stack_trace_from_pid(pid, use_jstack)))
         for thread in threads:
             thread_info = get_thread_info(thread)
             if thread_info.get('status', '').lower().strip() != 'runnable':
@@ -373,6 +388,8 @@ def parse_args():
                       dest="pretty", default=False, help="Force colors")
     parser.add_option("-g", "--grep", dest="grep", default=None,
                       help="Show only threads that match text", metavar="MATCH")
+    parser.add_option("-F", "--force", dest="jstack", action="store_false",
+                      default=True, help="Use gdb to forcibly obtain the stacktrace [DANGEROUS]")
     options, args = parser.parse_args()
 
     if os.isatty(sys.stdout.fileno()):
